/*
 * RejillaPanel.java
 *
 * Created on 22 de diciembre de 2006, 11:16
 */

package guitetris;
import data.*;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.KeyEvent;

/**
 *
 * @author  acu
 */
public class RejillaPanel extends javax.swing.JPanel {
    /**
     * Referencia al TetrisFrame donde se incluye este JPanel
     */
    private TetrisFrame frame;
    
    /**
     * Número de pixeles del ancho y alto de cada celda de
     * este tablero de juego
     */
    private int anchocelda=-1;
    
    
    /** Creates new form RejillaPanel */
    public RejillaPanel() {
        initComponents();
    }
    
    public RejillaPanel(TetrisFrame fr) {
        this();
        frame=fr;
    }
    
    /**
     * Dibuja cada una de las celdas de la matriz bidimensional de la Rejilla. Cada celda puede estar
     * ocupada por BLOQUE (muro exterior) o PIEZA (elemento de una Figura)
     * @param el Graphics donde se dibujará
     */
    public void dibujaRejilla(java.awt.Graphics g){
        int i,j;
        Rejilla rejilla=frame.getRejilla();
        
        int xoffset=(getWidth()-rejilla.getAnchura()*anchocelda)/2;
        for(i=0;i<rejilla.getAnchura();i++){
            for(j=0;j<rejilla.getAltura();j++){
                if(rejilla.getTipoCelda(i,j) == Rejilla.BLOQUE){
                    g.setColor(Color.BLACK);
                    g.drawRect(xoffset+i*anchocelda,j*anchocelda,anchocelda,anchocelda);
                } else if(rejilla.getTipoCelda(i,j) == Rejilla.PIEZA){
                    g.setColor(Color.YELLOW);
                    g.fillRect(xoffset+i*anchocelda,j*anchocelda,anchocelda,anchocelda);
                    g.setColor(Color.RED);
                    g.drawRect(xoffset+i*anchocelda,j*anchocelda,anchocelda,anchocelda);
                }
            }
        }
    }
    
    /**
     * Dibuja la Figura fig en el Graphics g pasado como parámetro (normalmente el asociado a este Canvas)
     * @param fig la Figura a dibujar
     * @param el Graphics donde se dibujará
     */
    void dibujaFigura(Figura fig,java.awt.Graphics g){
        if (fig!=null){
            Elemento elemento;
            Rejilla rejilla=frame.getRejilla();
            int xoffset=(getWidth()-rejilla.getAnchura()*anchocelda)/2+fig.getXOrigen()*anchocelda;
            int yoffset=fig.getYOrigen()*anchocelda;
            for(int i=0;i<fig.getNElements();i++){
                elemento=fig.getElementAt(i);
                g.setColor(Color.YELLOW);
                g.fillRect(xoffset+elemento.getColumna()*anchocelda,
                        yoffset+elemento.getFila()*anchocelda,anchocelda,anchocelda);
                g.setColor(Color.RED);
                g.drawRect(xoffset+elemento.getColumna()*anchocelda,
                        yoffset+elemento.getFila()*anchocelda,anchocelda,anchocelda);
            }
        }
    }
    
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);
        if(anchocelda==-1){
            anchocelda=Math.min(getWidth()/frame.getRejilla().getAnchura(),(getHeight()-10)/frame.getRejilla().getAltura());
        }
        g.setColor(Color.WHITE);
        g.fillRect(0,0,getWidth(),getHeight());
        dibujaRejilla(g);
        dibujaFigura(frame.getFigura(),g);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {

        setPreferredSize(new java.awt.Dimension(200, 400));
        addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyPressed(java.awt.event.KeyEvent evt) {
                formKeyPressed(evt);
            }
        });
        addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseEntered(java.awt.event.MouseEvent evt) {
                formMouseEntered(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 400, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(0, 300, Short.MAX_VALUE)
        );
    }// </editor-fold>//GEN-END:initComponents
    
    private void formMouseEntered(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_formMouseEntered
// TODO add your handling code here:
        requestFocus();
    }//GEN-LAST:event_formMouseEntered
    
    private void formKeyPressed(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_formKeyPressed
// TODO add your handling code here:
        if (evt.getKeyCode() == KeyEvent.VK_LEFT){
            if(!frame.getRejilla().seChoca(frame.getFigura(),Figura.IZQUIERDA)){
                frame.getFigura().mueve(Figura.IZQUIERDA);
                if(frame.getPanel()!=null)
                    frame.getPanel().repaint();
            }
        }else if (evt.getKeyCode() == KeyEvent.VK_RIGHT){
            if(!frame.getRejilla().seChoca(frame.getFigura(),Figura.DERECHA)){
                frame.getFigura().mueve(Figura.DERECHA);
                if(frame.getPanel()!=null)
                    frame.getPanel().repaint();
            }
        }else if (evt.getKeyCode() == KeyEvent.VK_UP){
            frame.getFigura().rotar(frame.getRejilla());
            if(frame.getPanel()!=null)
                frame.getPanel().repaint();
            
        }else if (evt.getKeyCode() == KeyEvent.VK_DOWN){
            if(!frame.getRejilla().seChoca(frame.getFigura(),Figura.ABAJO)){
                frame.getFigura().mueve(Figura.ABAJO);
                if(frame.getPanel()!=null)
                    frame.getPanel().repaint();
            }
        }
    }//GEN-LAST:event_formKeyPressed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    
}
